@@ -1,10 +1,12 @@
 from process_pose_data.honeycomb_io import *
 from process_pose_data.local_io import *
 from process_pose_data.filter import *
-from process_pose_data.analyze import *
+from process_pose_data.reconstruct import *
 from process_pose_data.visualize import *
 from process_pose_data.overlay import *
 from process_pose_data.track_poses import *
-from process_pose_data.identify_pose_tracks import *
+from process_pose_data.identify import *
 from process_pose_data.geom_render import *
 from process_pose_data.process import *
+
+__version__ = '3.0.0'
